@@ -1,6 +1,6 @@
 /* Initalization and related routines.
  *
- * IRC Services is copyright (c) 1996-2008 Andrew Church.
+ * IRC Services is copyright (c) 1996-2009 Andrew Church.
  *     E-mail: <achurch@achurch.org>
  * Parts written by Andrew Kempe and others.
  * This program is free but copyrighted software; see the file GPL.txt for
@@ -778,7 +778,7 @@ static int parse_options(int ac, char **av, int call_modules)
                         return -1;
                     }
                     free(LogFilename);
-                    LogFilename = s+4;
+                    LogFilename = sstrdup(s+4);
                 }
             } else if (strcmp(s, "debug") == 0) {
                 if (!call_modules)
@@ -929,6 +929,7 @@ int init(int ac, char **av)
     parse_options(ac, av, 0);
 
     /* Open logfile, and complain if we couldn't. */
+    set_logfile(LogFilename);
     if (!open_log()) {
         openlog_errno = errno;
         if (started_from_term) {
@@ -1188,6 +1189,7 @@ int reconfigure(void)
             " effect until restart");
     if (strcmp(LogFilename, old_LogFilename) != 0) {
         log("reconfigure: LogFilename changed, closing log file");
+        set_logfile(LogFilename);
         if (reopen_log()) {
             log("reconfigure: LogFilename changed, writing to new log file");
         } else {
@@ -1282,10 +1284,6 @@ int reconfigure(void)
 
 void cleanup(void)
 {
-    static char lognamebuf[BUFSIZE];
-    char *lognamesave;
-    int i;
-
     if (!*quitmsg)
         strbcpy(quitmsg, "Terminating, reason unknown");
     log("%s", quitmsg);
@@ -1312,16 +1310,7 @@ void cleanup(void)
     unregister_callback(cb_introduce_user);
     unregister_callback(cb_cmdline);
     modules_cleanup();
-    /* Save a copy of LogFilename, because the log routines still need it */
-    strbcpy(lognamebuf, LogFilename);
-    for (i = 0; strcmp(main_directives[i].name,"LogFilename") != 0; i++)
-        ;
-    lognamesave = main_directives[i].params[0].prev.ptrval;
-    main_directives[i].params[0].prev.ptrval = lognamebuf;
-    deconfigure(main_directives);
-    free(lognamesave);
     close_log();
-    /* Leave LogFilename set, just in case we need to write to the log again */
 }
 
 /*************************************************************************/
