@@ -1,6 +1,6 @@
 /* Bahamut protocol module for IRC Services.
  *
- * IRC Services is copyright (c) 1996-2008 Andrew Church.
+ * IRC Services is copyright (c) 1996-2009 Andrew Church.
  *     E-mail: <achurch@achurch.org>
  * Parts written by Andrew Kempe and others.
  * This program is free but copyrighted software; see the file GPL.txt for
@@ -30,6 +30,9 @@ static Module *module_chanserv;
 
 static char *NetworkDomain = NULL;
 
+static int32 usermode_secure = 0;       /* +S */
+static int32 chanmode_secure_only = 0;  /* +S */
+
 /*************************************************************************/
 /***************** Local interface to external routines ******************/
 /*************************************************************************/
@@ -49,6 +52,9 @@ static char **p_s_ChanServ = &ServerName;
 /************************** User/channel modes ***************************/
 /*************************************************************************/
 
+#define MI_SECURE       0x01000000  /* Usermode for users on secure conn's */
+#define MI_SECURE_ONLY  0x01000000  /* Chanmode for secure-user-only chans */
+
 struct modedata_init {
     uint8 mode;
     ModeData data;
@@ -61,6 +67,8 @@ static const struct modedata_init new_usermodes[] = {
                                 /* Registered nick */
     {'a', {0x00000040}},        /* Services admin */
     {'A', {0x00000080}},        /* Server admin */
+    {'S', {0x00000100,0,0,0,MI_SECURE}},
+                                /* Using secure connection */
 };
 
 static const struct modedata_init new_chanmodes[] = {
@@ -73,6 +81,8 @@ static const struct modedata_init new_chanmodes[] = {
                                 /* Only opers can join channel */
     {'M', {0x02000000,0,0}},    /* Moderated to unregged nicks */
     {'j', {0x04000000,1,0}},    /* /join rate limit */
+    {'S', {0x08000000,0,0,0,MI_SECURE_ONLY}},
+                                /* Only secure (+S) users allowed */
     {'e', {0x80000000,1,1,0,MI_MULTIPLE}},
                                 /* Ban exceptions */
     {'I', {0x80000000,1,1,0,MI_MULTIPLE}},
@@ -86,10 +96,16 @@ static void init_modes(void)
 {
     int i;
 
-    for (i = 0; i < lenof(new_usermodes); i++)
+    for (i = 0; i < lenof(new_usermodes); i++) {
         usermodes[new_usermodes[i].mode] = new_usermodes[i].data;
-    for (i = 0; i < lenof(new_chanmodes); i++)
+        if (new_usermodes[i].data.info & MI_SECURE)
+            usermode_secure |= new_usermodes[i].data.flag;
+    }
+    for (i = 0; i < lenof(new_chanmodes); i++) {
         chanmodes[new_chanmodes[i].mode] = new_chanmodes[i].data;
+        if (new_chanmodes[i].data.info & MI_SECURE_ONLY)
+            chanmode_secure_only |= new_chanmodes[i].data.flag;
+    }
     for (i = 0; i < lenof(new_chanusermodes); i++)
         chanusermodes[new_chanusermodes[i].mode] = new_chanusermodes[i].data;
 
@@ -675,6 +691,32 @@ static int do_check_modes(Channel *c, ChannelInfo *ci, int add, int32 flag)
 
 /*************************************************************************/
 
+/*
+ * Callback for ChanServ channel entrance checking; we enforce the
+ * secure-only (+S) channel mode here.
+ */
+
+static int do_check_kick(User *user, const char *chan, ChannelInfo *ci,
+                         char **mask_ret, const char **reason_ret)
+{
+    /* Retrieve the channel's Channel record, if present */
+    Channel *c = get_channel(chan);
+
+    /* Don't let users not on secure connections (-S) into +S channels */
+    if ((((c?c->mode:0) | (ci?ci->mlock.on:0)) & chanmode_secure_only)
+     && !(user->mode & usermode_secure)
+    ) {
+        *mask_ret = create_mask(user, 1);
+        *reason_ret = getstring(user->ngi, CHAN_NOT_ALLOWED_TO_JOIN);
+        return 1;
+    }
+
+    /* Let other processing continue as usual */
+    return 0;
+}
+
+/*************************************************************************/
+
 static int do_set_mlock(User *u, ChannelInfo *ci, int mode, int add, char **av)
 {
     if (!mode) {
@@ -836,6 +878,8 @@ static int do_load_module(Module *mod, const char *modname)
             p_s_ChanServ = &ServerName;
         if (!add_callback(mod, "check_modes", do_check_modes))
             module_log("Unable to add ChanServ check_modes callback");
+        if (!add_callback(mod, "check_kick", do_check_kick))
+            module_log("Unable to add ChanServ check_kick callback");
         if (!add_callback(mod, "SET MLOCK", do_set_mlock))
             module_log("Unable to add ChanServ SET MLOCK callback");
     }
@@ -852,8 +896,6 @@ static int do_unload_module(Module *mod)
     } else if (mod == module_chanserv) {
         module_chanserv = NULL;
         p_s_ChanServ = &ServerName;
-        remove_callback(mod, "SET MLOCK", do_set_mlock);
-        remove_callback(mod, "check_modes", do_check_modes);
     }
     return 0;
 }
