@@ -1,6 +1,6 @@
 /* solid-ircd protocol module for IRC Services.
  *
- * IRC Services is copyright (c) 1996-2008 Andrew Church.
+ * IRC Services is copyright (c) 1996-2009 Andrew Church.
  *     E-mail: <achurch@achurch.org>
  * Parts written by Andrew Kempe and others.
  * This program is free but copyrighted software; see the file GPL.txt for
@@ -840,8 +840,6 @@ static int do_unload_module(Module *mod)
     } else if (mod == module_chanserv) {
         module_chanserv = NULL;
         p_s_ChanServ = &ServerName;
-        remove_callback(mod, "SET MLOCK", do_set_mlock);
-        remove_callback(mod, "check_modes", do_check_modes);
     }
     return 0;
 }
