@@ -1,7 +1,7 @@
 /* Statistics generation (StatServ) main module.
  * Based on code by Andrew Kempe (TheShadow).
  *
- * IRC Services is copyright (c) 1996-2008 Andrew Church.
+ * IRC Services is copyright (c) 1996-2009 Andrew Church.
  *     E-mail: <achurch@achurch.org>
  * Parts written by Andrew Kempe and others.
  * This program is free but copyrighted software; see the file GPL.txt for
@@ -105,10 +105,26 @@ static void clean_dbtables(void)
 
 /*************************************************************************/
 
+/* Database load helper: if the server was online when the databas was last
+ * saved, set t_quit to a time just before Services started up, so servers
+ * are never seen as online until we actually receive the SERVER message.
+ */
+static void db_put_t_quit(void *record, const void *value)
+{
+    ServerStats *ss = (ServerStats *)record;
+    if (SS_IS_ONLINE(ss)) {
+        ss->t_quit = time(NULL) - 1;
+        if (SS_IS_ONLINE(ss)) {  /* Just in case */
+            ss->t_quit = ss->t_join + 1;
+        }
+    }
+}
+
 static DBField stat_servers_dbfields[] = {
     { "name",         DBTYPE_STRING, offsetof(ServerStats,name) },
     { "t_join",       DBTYPE_TIME,   offsetof(ServerStats,t_join) },
-    { "t_quit",       DBTYPE_TIME,   offsetof(ServerStats,t_quit) },
+    { "t_quit",       DBTYPE_TIME,   offsetof(ServerStats,t_quit),
+          .put = db_put_t_quit},
     { "quit_message", DBTYPE_STRING, offsetof(ServerStats,quit_message) },
     { NULL }
 };
@@ -251,7 +267,7 @@ static void do_servers(User *u)
                 && (!ss_lastquit
                     || ss->t_quit > ss_lastquit->t_quit))
                 ss_lastquit = ss;
-            if (ss->t_join > ss->t_quit)
+            if (SS_IS_ONLINE(ss))
                 onlinecount++;
         }
 
@@ -276,7 +292,7 @@ static void do_servers(User *u)
             if (mask && !match_wild_nocase(mask, ss->name))
                 continue;
             matchcount++;
-            if (ss->t_join < ss->t_quit)
+            if (!SS_IS_ONLINE(ss))
                 continue;
             count++;
             notice_lang(s_StatServ, u, STAT_SERVERS_LIST_FORMAT,
@@ -307,10 +323,7 @@ static void do_servers(User *u)
             nservers++;
             if (mask && !match_wild_nocase(mask, ss->name))
                 continue;
-            if (ss->t_join >= ss->t_quit)
-                is_online = 1;
-            else
-                is_online = 0;
+            is_online = SS_IS_ONLINE(ss);
             if (limitto && !((is_online && limitto == 1) ||
                              (!is_online && limitto == 2)))
                 continue;
@@ -356,7 +369,7 @@ static void do_servers(User *u)
             syntax_error(s_StatServ, u, "SERVERS", STAT_SERVERS_DELETE_SYNTAX);
         } else if (!(ss = get_serverstats(mask))) {
             notice_lang(s_StatServ, u, SERV_X_NOT_FOUND, mask);
-        } else if (ss->t_join > ss->t_quit) {
+        } else if (SS_IS_ONLINE(ss)) {
             notice_lang(s_StatServ, u, STAT_SERVERS_REMOVE_SERV_FIRST, mask);
         } else {
             del_serverstats(ss);
@@ -364,6 +377,55 @@ static void do_servers(User *u)
             notice_lang(s_StatServ, u, STAT_SERVERS_DELETE_DONE, mask);
         }
 
+    } else if (stricmp(cmd, "COPY") == 0) {
+        const char *newname = strtok(NULL, " ");
+        ServerStats *newss;
+        if (!mask || !newname) {
+            syntax_error(s_StatServ, u, "SERVERS", STAT_SERVERS_COPY_SYNTAX);
+        } else if (!(ss = get_serverstats(mask))) {
+            notice_lang(s_StatServ, u, SERV_X_NOT_FOUND, mask);
+        } else if ((newss = get_serverstats(newname)) != NULL) {
+            put_serverstats(newss);
+            notice_lang(s_StatServ, u, STAT_SERVERS_SERVER_EXISTS, newname);
+        } else {
+            newss = new_serverstats(newname);
+            newss->t_join = ss->t_join;
+            newss->t_quit = ss->t_quit;
+            if (ss->quit_message) {
+                newss->quit_message = sstrdup(ss->quit_message);
+            }
+            add_serverstats(newss);
+            put_serverstats(newss);
+            notice_lang(s_StatServ, u, STAT_SERVERS_COPY_DONE, mask, newname);
+        }
+
+    } else if (stricmp(cmd, "RENAME") == 0) {
+        const char *newname = strtok(NULL, " ");
+        ServerStats *newss;
+        if (!mask || !newname) {
+            syntax_error(s_StatServ, u, "SERVERS", STAT_SERVERS_RENAME_SYNTAX);
+        } else if (!(ss = get_serverstats(mask))) {
+            notice_lang(s_StatServ, u, SERV_X_NOT_FOUND, mask);
+        } else if ((newss = get_serverstats(newname)) != NULL) {
+            put_serverstats(newss);
+            notice_lang(s_StatServ, u, STAT_SERVERS_SERVER_EXISTS, newname);
+        } else if (SS_IS_ONLINE(ss)) {
+            notice_lang(s_StatServ, u, STAT_SERVERS_REMOVE_SERV_FIRST, mask);
+        } else {
+            newss = new_serverstats(newname);
+            newss->t_join = ss->t_join;
+            newss->t_quit = ss->t_quit;
+            if (ss->quit_message) {
+                newss->quit_message = sstrdup(ss->quit_message);
+            }
+            del_serverstats(ss);
+            ss = NULL;
+            add_serverstats(newss);
+            put_serverstats(newss);
+            notice_lang(s_StatServ, u, STAT_SERVERS_RENAME_DONE,
+                        mask, newname);
+        }
+
     } else {
         syntax_error(s_StatServ, u, "SERVERS", STAT_SERVERS_SYNTAX);
     }
@@ -427,7 +489,7 @@ void free_serverstats(ServerStats *ss)
 /************************** Callback routines ****************************/
 /*************************************************************************/
 
-/* Handle a server joining */
+/* Handle a server joining. */
 
 static int stats_do_server(Server *server)
 {
@@ -454,7 +516,7 @@ static int stats_do_server(Server *server)
 
 /*************************************************************************/
 
-/* Handle a server quitting */
+/* Handle a server quitting. */
 
 static int stats_do_squit(Server *server, const char *quit_message)
 {
@@ -470,7 +532,7 @@ static int stats_do_squit(Server *server, const char *quit_message)
 
 /*************************************************************************/
 
-/* Handle a user joining */
+/* Handle a user joining. */
 
 static int stats_do_newuser(User *user)
 {
@@ -481,7 +543,7 @@ static int stats_do_newuser(User *user)
 
 /*************************************************************************/
 
-/* Handle a user quitting */
+/* Handle a user quitting. */
 
 static int stats_do_quit(User *user)
 {
@@ -501,7 +563,7 @@ static int stats_do_quit(User *user)
 
 /*************************************************************************/
 
-/* Handle a user mode change */
+/* Handle a user mode change. */
 
 static int stats_do_umode(User *user, int modechar, int add)
 {
